First attempt to make console obey Qemu's -nographic flag.

Index: openbios-quilt/arch/sparc32/aoutload.c
===================================================================
--- openbios-quilt.orig/arch/sparc32/aoutload.c	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/arch/sparc32/aoutload.c	2006-06-17 11:49:54.000000000 +0000
@@ -126,14 +126,9 @@
 
 #if 1
     {
-        extern unsigned int qemu_mem_size;
-        extern char boot_device;
-        void *init_openprom(unsigned long memsize, const char *cmdline, char boot_device);
-
+        extern const void *romvec;
         int (*entry)(const void *romvec, int p2, int p3, int p4, int p5);
-        const void *romvec;
 
-        romvec = init_openprom(qemu_mem_size, cmdline, boot_device);
         entry = (void *) addr_fixup(start);
         image_retval = entry(romvec, 0, 0, 0, 0);
     }
Index: openbios-quilt/arch/sparc32/boot.c
===================================================================
--- openbios-quilt.orig/arch/sparc32/boot.c	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/arch/sparc32/boot.c	2006-06-17 11:49:54.000000000 +0000
@@ -21,6 +21,10 @@
 uint32_t cmdline;
 uint32_t cmdline_size;
 char boot_device;
+char nographic;
+const void *romvec;
+extern unsigned int qemu_mem_size;
+void *init_openprom(unsigned long memsize, const char *cmdline, char boot_device, int nographic);
 
 void boot(void)
 {
@@ -28,14 +32,10 @@
         char altpath[256];
 	
         if (kernel_size) {
-            extern unsigned int qemu_mem_size;
-            void *init_openprom(unsigned long memsize, const char *cmdline, char boot_device);
-
             int (*entry)(const void *romvec, int p2, int p3, int p4, int p5);
-            const void *romvec;
 
             printk("[sparc] Kernel already loaded\n");
-            romvec = init_openprom(qemu_mem_size, (void *)cmdline, boot_device);
+            romvec = init_openprom(qemu_mem_size, (void *)cmdline, boot_device, nographic);
             entry = (void *) kernel_image;
             entry(romvec, 0, 0, 0, 0);
         }
@@ -73,6 +73,8 @@
 		printk("without parameters.\n");
 
 
+        romvec = init_openprom(qemu_mem_size, (void *)param, boot_device, nographic);
+
 	if (elf_load(&sys_info, path, param) == LOADER_NOT_SUPPORT)
             if (linux_load(&sys_info, path, param) == LOADER_NOT_SUPPORT)
                 if (aout_load(&sys_info, path, param) == LOADER_NOT_SUPPORT) {
Index: openbios-quilt/arch/sparc32/elfload.c
===================================================================
--- openbios-quilt.orig/arch/sparc32/elfload.c	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/arch/sparc32/elfload.c	2006-06-17 11:49:54.000000000 +0000
@@ -383,14 +383,9 @@
 
 #if 1
     {
-        extern unsigned int qemu_mem_size;
-        extern char boot_device;
-        void *init_openprom(unsigned long memsize, const char *cmdline, char boot_device);
-
+        extern const void *romvec;
         int (*entry)(const void *romvec, int p2, int p3, int p4, int p5);
-        const void *romvec;
 
-        romvec = init_openprom(qemu_mem_size, cmdline, boot_device);
         entry = (void *) addr_fixup(ehdr.e_entry);
         image_retval = entry(romvec, 0, 0, 0, 0);
     }
Index: openbios-quilt/arch/sparc32/romvec.c
===================================================================
--- openbios-quilt.orig/arch/sparc32/romvec.c	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/arch/sparc32/romvec.c	2006-06-17 11:49:54.000000000 +0000
@@ -444,8 +444,10 @@
 }
 
 void *
-init_openprom(unsigned long memsize, const char *cmdline, char boot_device)
+init_openprom(unsigned long memsize, const char *cmdline, char boot_device, int nographic)
 {
+    const char *stdin, *stdout;
+
     free_ram = va2pa((int)&_data) - PAGE_SIZE;
 
     ptphys = totphys;
@@ -538,15 +540,51 @@
     romvec0.pv_v2bootargs.fd_stdin = &obp_fd_stdin;
     romvec0.pv_v2bootargs.fd_stdout = &obp_fd_stdout;
 
-    push_str("/builtin/console");
+#if 0
+    if (nographic) {
+        obp_stdin = PROMDEV_TTYA;
+        obp_stdout = PROMDEV_TTYA;
+        stdin = "/obio/zs@0,100000:a";
+        stdout = "/obio/zs@0,100000:a";
+    } else {
+        obp_stdin = PROMDEV_KBD;
+        obp_stdout = PROMDEV_SCREEN;
+        stdin = "/obio/zs@0,0";
+        stdout = "/iommu/sbus/SUNW,tcx";
+    }
+
+    push_str(stdin);
     fword("open-dev");
     obp_fd_stdin = POP();
-    push_str("/builtin/console");
+    push_str(stdout);
     fword("open-dev");
     obp_fd_stdout = POP();
-    
+#else
     obp_stdin = PROMDEV_TTYA;
     obp_stdout = PROMDEV_TTYA;
+    stdin = "/obio/zs@0,100000:a";
+    stdout = "/obio/zs@0,100000:a";
+
+    push_str("/builtin/console");
+    fword("open-dev");
+    obp_fd_stdin = POP();
+    push_str("/builtin/console");
+    fword("open-dev");
+    obp_fd_stdout = POP();
+#endif
+    push_str("/");
+    fword("find-device");
+    push_str(stdin);
+    fword("encode-string");
+    push_str("stdin-path");
+    fword("encode-string");
+    fword("property");
+
+    push_str(stdout);
+    fword("encode-string");
+    push_str("stdout-path");
+    fword("encode-string");
+    fword("property");
 
     romvec0.v3_cpustart = obp_cpustart;
     romvec0.v3_cpustop = obp_cpustop;
Index: openbios-quilt/arch/sparc32/tree.fs
===================================================================
--- openbios-quilt.orig/arch/sparc32/tree.fs	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/arch/sparc32/tree.fs	2006-06-17 11:49:54.000000000 +0000
@@ -9,10 +9,6 @@
   " SUNW,501-3059" encode-string " model" property
   h# 0a21fe80 encode-int " clock-frequency" property
   
-  " /obio/zs@0,100000:a" encode-string " stdin-path" property
-  " /obio/zs@0,100000:a" encode-string " stdout-path" property
-
-  
   : encode-unit encode-unit-sbus ;
   : decode-unit decode-unit-sbus ;
 
Index: openbios-quilt/drivers/obio.c
===================================================================
--- openbios-quilt.orig/drivers/obio.c	2006-06-17 11:42:14.000000000 +0000
+++ openbios-quilt/drivers/obio.c	2006-06-17 11:49:54.000000000 +0000
@@ -25,6 +25,13 @@
 			path, name##_m, sizeof(name##_m)/sizeof(method_t)); \
 	} while(0)
 
+#define REGISTER_NODE_METHODS( name, path )   do {                      \
+        const char *paths[1];                                                  \
+                                                                        \
+        paths[0] = path;                                                \
+        bind_node( name##_flags_, name##_size_,                         \
+                   paths, 1, name##_m, sizeof(name##_m)/sizeof(method_t)); \
+    } while(0)
 
 /* DECLARE data structures for the nodes.  */
 DECLARE_UNNAMED_NODE( ob_obio, INSTALL_OPEN, sizeof(int) );
@@ -88,10 +95,37 @@
     fword("property");
 }
 
+#if 0
+static void
+zs_open(void)
+{
+    RET ( -1 );
+}
 
 static void
+zs_close(void)
+{
+}
+
+static void
+zs_read(void)
+{
+}
+
+DECLARE_UNNAMED_NODE(zs, INSTALL_OPEN, sizeof(void *));
+
+NODE_METHODS(zs) = {
+	{ "open",               zs_open              },
+	{ "close",              zs_close             },
+	{ "read",               zs_read              },
+};
+
+#endif
+static void
 ob_zs_init(unsigned long base, unsigned long offset, int intr, int slave, int keyboard)
 {
+    char nodebuff[256];
+
     ob_new_obio_device("zs", "serial");
 
     ob_reg(base, offset, ZS_REGS, 1);
@@ -116,6 +150,10 @@
     ob_intr(intr);
 
     fword("finish-device");
+#if 0
+    sprintf(nodebuff, "/obio/zs@0,%x", offset);
+    REGISTER_NODE_METHODS(zs, nodebuff);
+#endif
 }
 
 static char *nvram;
@@ -147,6 +185,7 @@
     extern uint32_t cmdline;
     extern uint32_t cmdline_size;
     extern char boot_device;
+    extern char nographic;
 
     unsigned int i;
 
@@ -171,6 +210,7 @@
     cmdline = nv_info.cmdline;
     cmdline_size = nv_info.cmdline_size;
     boot_device = nv_info.boot_device;
+    nographic = nv_info.nographic;
 
     push_str("mk48t08");
     fword("model");
